@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2022 NVIDIA Corporation
+    Copyright (c) 2015-2023 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -29,8 +29,9 @@
 #include "uvm_tracker.h"
 #include "uvm_gpu.h"
 #include "uvm_va_space_mm.h"
+#include "uvm_processors.h"
 
-bool uvm_is_valid_vma_range(struct mm_struct *mm, NvU64 start, NvU64 length)
+static bool uvm_is_valid_vma_range(struct mm_struct *mm, NvU64 start, NvU64 length)
 {
     const NvU64 end = start + length;
     struct vm_area_struct *vma;
@@ -44,15 +45,15 @@ bool uvm_is_valid_vma_range(struct mm_struct *mm, NvU64 start, NvU64 length)
         if (vma->vm_end >= end)
             return true;
         start = vma->vm_end;
-        vma = vma->vm_next;
+        vma = find_vma_intersection(mm, start, end);
     }
 
     return false;
 }
 
-NV_STATUS uvm_api_range_type_check(uvm_va_space_t *va_space, struct mm_struct *mm, NvU64 base, NvU64 length)
+uvm_api_range_type_t uvm_api_range_type_check(uvm_va_space_t *va_space, struct mm_struct *mm, NvU64 base, NvU64 length)
 {
-    uvm_va_range_t *va_range, *va_range_last;
+    uvm_va_range_managed_t *managed_range, *managed_range_last;
     const NvU64 last_address = base + length - 1;
 
     if (mm)
@@ -61,35 +62,37 @@ NV_STATUS uvm_api_range_type_check(uvm_va_space_t *va_space, struct mm_struct *m
     uvm_assert_rwsem_locked(&va_space->lock);
 
     if (uvm_api_range_invalid(base, length))
-        return NV_ERR_INVALID_ADDRESS;
+        return UVM_API_RANGE_TYPE_INVALID;
 
     // Check if passed interval overlaps with any VA range.
     if (uvm_va_space_range_empty(va_space, base, last_address)) {
         if (g_uvm_global.ats.enabled &&
             uvm_va_space_pageable_mem_access_supported(va_space) &&
             mm &&
-            uvm_is_valid_vma_range(mm, base, length))
-            return NV_WARN_NOTHING_TO_DO;
-        else if (uvm_hmm_is_enabled(va_space) &&
-                 mm &&
-                 uvm_is_valid_vma_range(mm, base, length))
-            return NV_OK;
-        else
-            return NV_ERR_INVALID_ADDRESS;
+            uvm_is_valid_vma_range(mm, base, length)) {
+
+            return UVM_API_RANGE_TYPE_ATS;
+        }
+        else if (uvm_hmm_is_enabled(va_space) && mm && uvm_is_valid_vma_range(mm, base, length)) {
+            return UVM_API_RANGE_TYPE_HMM;
+        }
+        else {
+            return UVM_API_RANGE_TYPE_INVALID;
+        }
     }
 
-    va_range_last = NULL;
+    managed_range_last = NULL;
 
-    uvm_for_each_managed_va_range_in_contig(va_range, va_space, base, last_address)
-        va_range_last = va_range;
+    uvm_for_each_va_range_managed_in_contig(managed_range, va_space, base, last_address)
+        managed_range_last = managed_range;
 
     // Check if passed interval overlaps with an unmanaged VA range, or a
-    // sub-interval not tracked by a VA range
-    if (!va_range_last || va_range_last->node.end < last_address)
-        return NV_ERR_INVALID_ADDRESS;
+    // sub-interval not tracked by a managed range
+    if (!managed_range_last || managed_range_last->va_range.node.end < last_address)
+        return UVM_API_RANGE_TYPE_INVALID;
 
-    // Passed interval is fully covered by managed VA ranges
-    return NV_OK;
+    // Passed interval is fully covered by managed ranges
+    return UVM_API_RANGE_TYPE_MANAGED;
 }
 
 static NV_STATUS split_as_needed(uvm_va_space_t *va_space,
@@ -97,11 +100,12 @@ static NV_STATUS split_as_needed(uvm_va_space_t *va_space,
                                  uvm_va_policy_is_split_needed_t split_needed_cb,
                                  void *data)
 {
+    uvm_va_range_managed_t *managed_range;
     uvm_va_range_t *va_range;
 
     UVM_ASSERT(PAGE_ALIGNED(addr));
 
-    // Look for UVM managed allocations first, then look for HMM policies.
+    // Look for managed allocations first, then look for HMM policies.
     va_range = uvm_va_range_find(va_space, addr);
     if (!va_range)
         return uvm_hmm_split_as_needed(va_space, addr, split_needed_cb, data);
@@ -110,12 +114,14 @@ static NV_STATUS split_as_needed(uvm_va_space_t *va_space,
     if (addr == va_range->node.start)
         return NV_OK;
 
+    managed_range = uvm_va_range_to_managed_or_null(va_range);
+
     // Only managed ranges can be split.
-    if (va_range->type != UVM_VA_RANGE_TYPE_MANAGED)
+    if (!managed_range)
         return NV_ERR_INVALID_ADDRESS;
 
-    if (split_needed_cb(uvm_va_range_get_policy(va_range), data))
-        return uvm_va_range_split(va_range, addr - 1, NULL);
+    if (split_needed_cb(&managed_range->policy, data))
+        return uvm_va_range_split(managed_range, addr - 1, NULL);
 
     return NV_OK;
 }
@@ -141,23 +147,29 @@ static NV_STATUS split_span_as_needed(uvm_va_space_t *va_space,
     return split_as_needed(va_space, end_addr, split_needed_cb, data);
 }
 
-static bool preferred_location_is_split_needed(uvm_va_policy_t *policy, void *data)
+typedef struct
 {
     uvm_processor_id_t processor_id;
+    int nid;
+} preferred_location_split_params_t;
 
-    UVM_ASSERT(data);
+static bool preferred_location_is_split_needed(const uvm_va_policy_t *policy, void *data)
+{
+    preferred_location_split_params_t *params = (preferred_location_split_params_t *)data;
 
-    processor_id = *(uvm_processor_id_t*)data;
-    return !uvm_id_equal(processor_id, policy->preferred_location);
+    UVM_ASSERT(params);
+
+    return !uvm_va_policy_preferred_location_equal(policy, params->processor_id, params->nid);
 }
 
 static NV_STATUS preferred_location_unmap_remote_pages(uvm_va_block_t *va_block,
-                                                       uvm_va_block_context_t *va_block_context)
+                                                       uvm_va_block_context_t *va_block_context,
+                                                       uvm_va_block_region_t region)
 {
     NV_STATUS status = NV_OK;
     NV_STATUS tracker_status;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
-    uvm_va_policy_t *policy = va_block_context->policy;
+    const uvm_va_policy_t *policy = uvm_va_policy_get_region(va_block, region);
     uvm_processor_id_t preferred_location = policy->preferred_location;
     uvm_va_space_t *va_space = uvm_va_block_get_va_space(va_block);
     const uvm_page_mask_t *mapped_mask;
@@ -173,7 +185,9 @@ static NV_STATUS preferred_location_unmap_remote_pages(uvm_va_block_t *va_block,
     mapped_mask = uvm_va_block_map_mask_get(va_block, preferred_location);
 
     if (uvm_processor_mask_test(&va_block->resident, preferred_location)) {
-        const uvm_page_mask_t *resident_mask = uvm_va_block_resident_mask_get(va_block, preferred_location);
+        const uvm_page_mask_t *resident_mask = uvm_va_block_resident_mask_get(va_block,
+                                                                              preferred_location,
+                                                                              NUMA_NO_NODE);
 
         if (!uvm_page_mask_andnot(&va_block_context->caller_page_mask, mapped_mask, resident_mask))
             goto done;
@@ -185,7 +199,7 @@ static NV_STATUS preferred_location_unmap_remote_pages(uvm_va_block_t *va_block,
     status = uvm_va_block_unmap(va_block,
                                 va_block_context,
                                 preferred_location,
-                                uvm_va_block_region_from_block(va_block),
+                                region,
                                 &va_block_context->caller_page_mask,
                                 &local_tracker);
 
@@ -200,13 +214,15 @@ done:
 }
 
 NV_STATUS uvm_va_block_set_preferred_location_locked(uvm_va_block_t *va_block,
-                                                     uvm_va_block_context_t *va_block_context)
+                                                     uvm_va_block_context_t *va_block_context,
+                                                     uvm_va_block_region_t region)
 {
     uvm_assert_mutex_locked(&va_block->lock);
 
-    uvm_va_block_mark_cpu_dirty(va_block);
+    if (!uvm_va_block_is_hmm(va_block))
+        uvm_va_block_mark_cpu_dirty(va_block);
 
-    return preferred_location_unmap_remote_pages(va_block, va_block_context);
+    return preferred_location_unmap_remote_pages(va_block, va_block_context, region);
 }
 
 static NV_STATUS preferred_location_set(uvm_va_space_t *va_space,
@@ -214,44 +230,48 @@ static NV_STATUS preferred_location_set(uvm_va_space_t *va_space,
                                         NvU64 base,
                                         NvU64 length,
                                         uvm_processor_id_t preferred_location,
-                                        uvm_va_range_t **first_va_range_to_migrate,
+                                        int preferred_cpu_nid,
+                                        uvm_va_range_managed_t **first_managed_range_to_migrate,
                                         uvm_tracker_t *out_tracker)
 {
-    uvm_va_range_t *va_range, *va_range_last;
+    uvm_va_range_managed_t *managed_range, *managed_range_last;
     const NvU64 last_address = base + length - 1;
     bool preferred_location_is_faultable_gpu = false;
+    preferred_location_split_params_t split_params;
     NV_STATUS status;
 
     uvm_assert_rwsem_locked_write(&va_space->lock);
 
     if (UVM_ID_IS_VALID(preferred_location)) {
-        *first_va_range_to_migrate = NULL;
+        *first_managed_range_to_migrate = NULL;
         preferred_location_is_faultable_gpu = UVM_ID_IS_GPU(preferred_location) &&
                                               uvm_processor_mask_test(&va_space->faultable_processors,
                                                                       preferred_location);
     }
 
+    split_params.processor_id = preferred_location;
+    split_params.nid = preferred_cpu_nid;
     status = split_span_as_needed(va_space,
                                   base,
                                   last_address + 1,
                                   preferred_location_is_split_needed,
-                                  &preferred_location);
+                                  &split_params);
     if (status != NV_OK)
         return status;
 
-    va_range_last = NULL;
-    uvm_for_each_managed_va_range_in_contig(va_range, va_space, base, last_address) {
+    managed_range_last = NULL;
+    uvm_for_each_va_range_managed_in_contig(managed_range, va_space, base, last_address) {
         bool found_non_migratable_interval = false;
 
-        va_range_last = va_range;
+        managed_range_last = managed_range;
 
         // If we didn't split the ends, check that they match
-        if (va_range->node.start < base || va_range->node.end > last_address)
-            UVM_ASSERT(uvm_id_equal(uvm_va_range_get_policy(va_range)->preferred_location, preferred_location));
+        if (managed_range->va_range.node.start < base || managed_range->va_range.node.end > last_address)
+            UVM_ASSERT(uvm_id_equal(managed_range->policy.preferred_location, preferred_location));
 
         if (UVM_ID_IS_VALID(preferred_location)) {
-            const NvU64 start = max(base, va_range->node.start);
-            const NvU64 end = min(last_address, va_range->node.end);
+            const NvU64 start = max(base, managed_range->va_range.node.start);
+            const NvU64 end = min(last_address, managed_range->va_range.node.end);
 
             found_non_migratable_interval = !uvm_range_group_all_migratable(va_space, start, end);
 
@@ -259,39 +279,52 @@ static NV_STATUS preferred_location_set(uvm_va_space_t *va_space,
                 return NV_ERR_INVALID_DEVICE;
         }
 
-        status = uvm_va_range_set_preferred_location(va_range, preferred_location, mm, out_tracker);
+        status = uvm_va_range_set_preferred_location(managed_range,
+                                                     preferred_location,
+                                                     preferred_cpu_nid,
+                                                     mm,
+                                                     out_tracker);
         if (status != NV_OK)
             return status;
 
-        // Return the first VA range that needs to be migrated so the caller
-        // function doesn't need to traverse the tree again
-        if (found_non_migratable_interval && (*first_va_range_to_migrate == NULL))
-            *first_va_range_to_migrate = va_range;
+        // Return the first managed range that needs to be migrated so the
+        // caller function doesn't need to traverse the tree again
+        if (found_non_migratable_interval && (*first_managed_range_to_migrate == NULL))
+            *first_managed_range_to_migrate = managed_range;
     }
 
-    if (va_range_last) {
-        UVM_ASSERT(va_range_last->node.end >= last_address);
+    if (managed_range_last) {
+        UVM_ASSERT(managed_range_last->va_range.node.end >= last_address);
         return NV_OK;
     }
 
-    return uvm_hmm_set_preferred_location(va_space, preferred_location, base, last_address);
+    if (!mm)
+        return NV_ERR_INVALID_ADDRESS;
+
+    return uvm_hmm_set_preferred_location(va_space,
+                                          preferred_location,
+                                          preferred_cpu_nid,
+                                          base,
+                                          last_address,
+                                          out_tracker);
 }
 
 NV_STATUS uvm_api_set_preferred_location(const UVM_SET_PREFERRED_LOCATION_PARAMS *params, struct file *filp)
 {
-    NV_STATUS status;
+    NV_STATUS status = NV_OK;
     NV_STATUS tracker_status;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
     uvm_va_space_t *va_space = uvm_va_space_get(filp);
-    uvm_va_range_t *va_range = NULL;
-    uvm_va_range_t *first_va_range_to_migrate = NULL;
+    uvm_va_range_managed_t *managed_range = NULL;
+    uvm_va_range_managed_t *first_managed_range_to_migrate = NULL;
     struct mm_struct *mm;
     uvm_processor_id_t preferred_location_id;
+    int preferred_cpu_nid = NUMA_NO_NODE;
     bool has_va_space_write_lock;
     const NvU64 start = params->requestedBase;
     const NvU64 length = params->length;
     const NvU64 end = start + length - 1;
-    bool range_is_ats = false;
+    uvm_api_range_type_t type;
 
     UVM_ASSERT(va_space);
 
@@ -299,18 +332,24 @@ NV_STATUS uvm_api_set_preferred_location(const UVM_SET_PREFERRED_LOCATION_PARAMS
     uvm_va_space_down_write(va_space);
     has_va_space_write_lock = true;
 
-    status = uvm_api_range_type_check(va_space, mm, start, length);
-    if (status != NV_OK) {
-        if (status != NV_WARN_NOTHING_TO_DO)
-            goto done;
-
-        status = NV_OK;
-        range_is_ats = true;
+    type = uvm_api_range_type_check(va_space, mm, start, length);
+    if (type == UVM_API_RANGE_TYPE_INVALID) {
+        status = NV_ERR_INVALID_ADDRESS;
+        goto done;
     }
 
     // If the CPU is the preferred location, we don't have to find the associated uvm_gpu_t
     if (uvm_uuid_is_cpu(&params->preferredLocation)) {
         preferred_location_id = UVM_ID_CPU;
+        preferred_cpu_nid = params->preferredCpuNumaNode;
+
+        if (preferred_cpu_nid != -1 &&
+            (!nv_numa_node_has_memory(preferred_cpu_nid) ||
+             !node_isset(preferred_cpu_nid, node_possible_map) ||
+             uvm_va_space_find_gpu_with_memory_node_id(va_space, preferred_cpu_nid))) {
+            status = NV_ERR_INVALID_ARGUMENT;
+            goto done;
+        }
     }
     else {
         // Translate preferredLocation into a live GPU ID, and check that this
@@ -330,31 +369,54 @@ NV_STATUS uvm_api_set_preferred_location(const UVM_SET_PREFERRED_LOCATION_PARAMS
 
     UVM_ASSERT(status == NV_OK);
 
-    // TODO: Bug 2098544: On ATS systems, honor the preferred location policy
-    // for system memory ranges instead of ignoring it.
-    if (range_is_ats)
+    // UvmSetPreferredLocation on non-ATS regions targets the VA range of the
+    // associated file descriptor, not the calling process. Since
+    // UvmSetPreferredLocation on ATS regions are handled in userspace,
+    // implementing the non-ATS behavior is not possible. So, return an error
+    // instead. Although the out of process case can be supported for HMM,
+    // return an error to make the API behavior consistent for all SAM regions.
+    if ((type != UVM_API_RANGE_TYPE_MANAGED) && (current->mm != mm)) {
+        status = NV_ERR_NOT_SUPPORTED;
         goto done;
+    }
 
-    status = preferred_location_set(va_space, mm, start, length, preferred_location_id, &first_va_range_to_migrate, &local_tracker);
+    // For ATS regions, let userspace handle it.
+    if (type == UVM_API_RANGE_TYPE_ATS) {
+        UVM_ASSERT(g_uvm_global.ats.enabled);
+        status = NV_WARN_NOTHING_TO_DO;
+        goto done;
+    }
+
+    status = preferred_location_set(va_space,
+                                    mm,
+                                    start,
+                                    length,
+                                    preferred_location_id,
+                                    preferred_cpu_nid,
+                                    &first_managed_range_to_migrate,
+                                    &local_tracker);
     if (status != NV_OK)
         goto done;
 
-    // No VA range to migrate, early exit
-    if (!first_va_range_to_migrate)
+    // No managed range to migrate, early exit
+    if (!first_managed_range_to_migrate)
         goto done;
 
     uvm_va_space_downgrade_write(va_space);
     has_va_space_write_lock = false;
 
-    // No need to check for holes in the VA ranges span here, this was checked by preferred_location_set
-    for (va_range = first_va_range_to_migrate; va_range; va_range = uvm_va_space_iter_next(va_range, end)) {
+    // No need to check for holes in the managed ranges span here, this was
+    // checked by preferred_location_set
+    for (managed_range = first_managed_range_to_migrate;
+         managed_range;
+         managed_range = uvm_va_space_iter_managed_next(managed_range, end)) {
         uvm_range_group_range_iter_t iter;
-        NvU64 cur_start = max(start, va_range->node.start);
-        NvU64 cur_end = min(end, va_range->node.end);
+        NvU64 cur_start = max(start, managed_range->va_range.node.start);
+        NvU64 cur_end = min(end, managed_range->va_range.node.end);
 
         uvm_range_group_for_each_migratability_in(&iter, va_space, cur_start, cur_end) {
             if (!iter.migratable) {
-                status = uvm_range_group_va_range_migrate(va_range, iter.start, iter.end, &local_tracker);
+                status = uvm_range_group_va_range_migrate(managed_range, iter.start, iter.end, &local_tracker);
                 if (status != NV_OK)
                     goto done;
             }
@@ -381,19 +443,39 @@ NV_STATUS uvm_api_unset_preferred_location(const UVM_UNSET_PREFERRED_LOCATION_PA
     uvm_va_space_t *va_space = uvm_va_space_get(filp);
     struct mm_struct *mm;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
+    uvm_api_range_type_t type;
 
     UVM_ASSERT(va_space);
 
     mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
-    status = uvm_api_range_type_check(va_space, mm, params->requestedBase, params->length);
+    type = uvm_api_range_type_check(va_space, mm, params->requestedBase, params->length);
+    if (type == UVM_API_RANGE_TYPE_INVALID) {
+        status = NV_ERR_INVALID_ADDRESS;
+        goto done;
+    }
 
-    if (status == NV_OK)
-        status = preferred_location_set(va_space, mm, params->requestedBase, params->length, UVM_ID_INVALID, NULL, &local_tracker);
-    else if (status == NV_WARN_NOTHING_TO_DO)
-        status = NV_OK;
+    if ((type != UVM_API_RANGE_TYPE_MANAGED) && (current->mm != mm)) {
+        status = NV_ERR_NOT_SUPPORTED;
+        goto done;
+    }
 
+    if (type == UVM_API_RANGE_TYPE_ATS) {
+        status = NV_WARN_NOTHING_TO_DO;
+        goto done;
+    }
+
+    status = preferred_location_set(va_space,
+                                    mm,
+                                    params->requestedBase,
+                                    params->length,
+                                    UVM_ID_INVALID,
+                                    NUMA_NO_NODE,
+                                    NULL,
+                                    &local_tracker);
+
+done:
     tracker_status = uvm_tracker_wait_deinit(&local_tracker);
 
     uvm_va_space_up_write(va_space);
@@ -401,10 +483,11 @@ NV_STATUS uvm_api_unset_preferred_location(const UVM_UNSET_PREFERRED_LOCATION_PA
     return status == NV_OK ? tracker_status : status;
 }
 
-static NV_STATUS va_block_set_accessed_by_locked(uvm_va_block_t *va_block,
-                                                 uvm_va_block_context_t *va_block_context,
-                                                 uvm_processor_id_t processor_id,
-                                                 uvm_tracker_t *out_tracker)
+NV_STATUS uvm_va_block_set_accessed_by_locked(uvm_va_block_t *va_block,
+                                              uvm_va_block_context_t *va_block_context,
+                                              uvm_processor_id_t processor_id,
+                                              uvm_va_block_region_t region,
+                                              uvm_tracker_t *out_tracker)
 {
     NV_STATUS status;
     NV_STATUS tracker_status;
@@ -414,7 +497,7 @@ static NV_STATUS va_block_set_accessed_by_locked(uvm_va_block_t *va_block,
     status = uvm_va_block_add_mappings(va_block,
                                        va_block_context,
                                        processor_id,
-                                       uvm_va_block_region_from_block(va_block),
+                                       region,
                                        NULL,
                                        UvmEventMapRemoteCausePolicy);
 
@@ -428,23 +511,25 @@ NV_STATUS uvm_va_block_set_accessed_by(uvm_va_block_t *va_block,
                                        uvm_processor_id_t processor_id)
 {
     uvm_va_space_t *va_space = uvm_va_block_get_va_space(va_block);
+    uvm_va_block_region_t region = uvm_va_block_region_from_block(va_block);
     NV_STATUS status;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
+    uvm_va_policy_t *policy = &va_block->managed_range->policy;
 
     UVM_ASSERT(!uvm_va_block_is_hmm(va_block));
 
-    va_block_context->policy = uvm_va_range_get_policy(va_block->va_range);
-
-    // Read duplication takes precedence over SetAccesedBy. Do not add mappings
+    // Read duplication takes precedence over SetAccessedBy. Do not add mappings
     // if read duplication is enabled.
-    if (uvm_va_policy_is_read_duplicate(va_block_context->policy, va_space))
+    if (uvm_va_policy_is_read_duplicate(policy, va_space))
         return NV_OK;
 
-    status = UVM_VA_BLOCK_LOCK_RETRY(va_block, NULL,
-                                     va_block_set_accessed_by_locked(va_block,
-                                                                     va_block_context,
-                                                                     processor_id,
-                                                                     &local_tracker));
+    status = UVM_VA_BLOCK_LOCK_RETRY(va_block,
+                                     NULL,
+                                     uvm_va_block_set_accessed_by_locked(va_block,
+                                                                         va_block_context,
+                                                                         processor_id,
+                                                                         region,
+                                                                         &local_tracker));
 
     // TODO: Bug 1767224: Combine all accessed_by operations into single tracker
     if (status == NV_OK)
@@ -460,7 +545,7 @@ typedef struct
     bool set_bit;
 } accessed_by_split_params_t;
 
-static bool accessed_by_is_split_needed(uvm_va_policy_t *policy, void *data)
+static bool accessed_by_is_split_needed(const uvm_va_policy_t *policy, void *data)
 {
     accessed_by_split_params_t *params = (accessed_by_split_params_t*)data;
 
@@ -476,26 +561,23 @@ static NV_STATUS accessed_by_set(uvm_va_space_t *va_space,
                                  bool set_bit)
 {
     uvm_processor_id_t processor_id = UVM_ID_INVALID;
-    uvm_va_range_t *va_range, *va_range_last;
     struct mm_struct *mm;
     const NvU64 last_address = base + length - 1;
-    bool range_is_sysmem = false;
     accessed_by_split_params_t split_params;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
-    NV_STATUS status;
+    NV_STATUS status = NV_OK;
     NV_STATUS tracker_status;
+    uvm_api_range_type_t type;
 
     UVM_ASSERT(va_space);
 
     mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
-    status = uvm_api_range_type_check(va_space, mm, base, length);
-    if (status != NV_OK) {
-        if (status != NV_WARN_NOTHING_TO_DO)
-            goto done;
-        status = NV_OK;
-        range_is_sysmem = true;
+    type = uvm_api_range_type_check(va_space, mm, base, length);
+    if (type == UVM_API_RANGE_TYPE_INVALID) {
+        status = NV_ERR_INVALID_ADDRESS;
+        goto done;
     }
 
     if (uvm_uuid_is_cpu(processor_uuid)) {
@@ -516,8 +598,10 @@ static NV_STATUS accessed_by_set(uvm_va_space_t *va_space,
         processor_id = gpu->id;
     }
 
-    if (range_is_sysmem)
+    if (type == UVM_API_RANGE_TYPE_ATS) {
+        status = NV_OK;
         goto done;
+    }
 
     split_params.processor_id = processor_id;
     split_params.set_bit = set_bit;
@@ -529,35 +613,42 @@ static NV_STATUS accessed_by_set(uvm_va_space_t *va_space,
     if (status != NV_OK)
         goto done;
 
-    va_range_last = NULL;
-    uvm_for_each_managed_va_range_in_contig(va_range, va_space, base, last_address) {
-        va_range_last = va_range;
+    if (type == UVM_API_RANGE_TYPE_MANAGED) {
+        uvm_va_range_managed_t *managed_range;
+        uvm_va_range_managed_t *managed_range_last = NULL;
 
-        // If we didn't split the ends, check that they match
-        if (va_range->node.start < base || va_range->node.end > last_address)
-            UVM_ASSERT(uvm_processor_mask_test(&uvm_va_range_get_policy(va_range)->accessed_by,
-                                               processor_id) == set_bit);
+        uvm_for_each_va_range_managed_in_contig(managed_range, va_space, base, last_address) {
+            managed_range_last = managed_range;
 
-        if (set_bit) {
-            status = uvm_va_range_set_accessed_by(va_range, processor_id, mm, &local_tracker);
-            if (status != NV_OK)
-                goto done;
+            // If we didn't split the ends, check that they match
+            if (managed_range->va_range.node.start < base || managed_range->va_range.node.end > last_address)
+                UVM_ASSERT(uvm_processor_mask_test(&managed_range->policy.accessed_by,
+                                                   processor_id) == set_bit);
+
+            if (set_bit) {
+                status = uvm_va_range_set_accessed_by(managed_range, processor_id, mm, &local_tracker);
+                if (status != NV_OK)
+                    goto done;
+            }
+            else {
+                uvm_va_range_unset_accessed_by(managed_range, processor_id, &local_tracker);
+            }
         }
-        else {
-            uvm_va_range_unset_accessed_by(va_range, processor_id, &local_tracker);
-        }
-    }
 
-    if (va_range_last) {
-        UVM_ASSERT(va_range_last->node.end >= last_address);
-        goto done;
+        UVM_ASSERT(managed_range_last);
+        UVM_ASSERT(managed_range_last->va_range.node.end >= last_address);
+    }
+    else {
+        // NULL mm case already filtered by uvm_api_range_type_check()
+        UVM_ASSERT(mm);
+        UVM_ASSERT(type == UVM_API_RANGE_TYPE_HMM);
+        status = uvm_hmm_set_accessed_by(va_space,
+                                         processor_id,
+                                         set_bit,
+                                         base,
+                                         last_address,
+                                         &local_tracker);
     }
-
-    status = uvm_hmm_set_accessed_by(va_space,
-                                     processor_id,
-                                     set_bit,
-                                     base,
-                                     last_address);
 
 done:
     tracker_status = uvm_tracker_wait_deinit(&local_tracker);
@@ -590,9 +681,12 @@ static NV_STATUS va_block_set_read_duplication_locked(uvm_va_block_t *va_block,
 
     uvm_assert_mutex_locked(&va_block->lock);
 
+    // Force CPU page residency to be on the preferred NUMA node.
+    va_block_context->make_resident.dest_nid = va_block->managed_range->policy.preferred_nid;
+
     for_each_id_in_mask(src_id, &va_block->resident) {
         NV_STATUS status;
-        uvm_page_mask_t *resident_mask = uvm_va_block_resident_mask_get(va_block, src_id);
+        uvm_page_mask_t *resident_mask = uvm_va_block_resident_mask_get(va_block, src_id, NUMA_NO_NODE);
 
         // Calling uvm_va_block_make_resident_read_duplicate will break all
         // SetAccessedBy and remote mappings
@@ -617,6 +711,9 @@ NV_STATUS uvm_va_block_set_read_duplication(uvm_va_block_t *va_block,
     NV_STATUS status;
     uvm_va_block_retry_t va_block_retry;
 
+    // TODO: Bug 3660922: need to implement HMM read duplication support.
+    UVM_ASSERT(!uvm_va_block_is_hmm(va_block));
+
     status = UVM_VA_BLOCK_LOCK_RETRY(va_block, &va_block_retry,
                                      va_block_set_read_duplication_locked(va_block,
                                                                           &va_block_retry,
@@ -634,9 +731,8 @@ static NV_STATUS va_block_unset_read_duplication_locked(uvm_va_block_t *va_block
     uvm_processor_id_t processor_id;
     uvm_va_block_region_t block_region = uvm_va_block_region_from_block(va_block);
     uvm_page_mask_t *break_read_duplication_pages = &va_block_context->caller_page_mask;
-    uvm_va_policy_t *policy = va_block_context->policy;
+    const uvm_va_policy_t *policy = &va_block->managed_range->policy;
     uvm_processor_id_t preferred_location = policy->preferred_location;
-    uvm_processor_mask_t accessed_by = policy->accessed_by;
 
     uvm_assert_mutex_locked(&va_block->lock);
 
@@ -646,7 +742,7 @@ static NV_STATUS va_block_unset_read_duplication_locked(uvm_va_block_t *va_block
     // If preferred_location is set and has resident copies, give it preference
     if (UVM_ID_IS_VALID(preferred_location) &&
         uvm_processor_mask_test(&va_block->resident, preferred_location)) {
-        uvm_page_mask_t *resident_mask = uvm_va_block_resident_mask_get(va_block, preferred_location);
+        uvm_page_mask_t *resident_mask = uvm_va_block_resident_mask_get(va_block, preferred_location, NUMA_NO_NODE);
         bool is_mask_empty = !uvm_page_mask_and(break_read_duplication_pages,
                                                 &va_block->read_duplicated_pages,
                                                 resident_mask);
@@ -674,7 +770,7 @@ static NV_STATUS va_block_unset_read_duplication_locked(uvm_va_block_t *va_block
         if (uvm_id_equal(processor_id, preferred_location))
             continue;
 
-        resident_mask = uvm_va_block_resident_mask_get(va_block, processor_id);
+        resident_mask = uvm_va_block_resident_mask_get(va_block, processor_id, NUMA_NO_NODE);
         is_mask_empty = !uvm_page_mask_and(break_read_duplication_pages,
                                            &va_block->read_duplicated_pages,
                                            resident_mask);
@@ -695,11 +791,12 @@ static NV_STATUS va_block_unset_read_duplication_locked(uvm_va_block_t *va_block
     }
 
     // 2- Re-establish SetAccessedBy mappings
-    for_each_id_in_mask(processor_id, &accessed_by) {
-        status = va_block_set_accessed_by_locked(va_block,
-                                                 va_block_context,
-                                                 processor_id,
-                                                 out_tracker);
+    for_each_id_in_mask(processor_id, &policy->accessed_by) {
+        status = uvm_va_block_set_accessed_by_locked(va_block,
+                                                     va_block_context,
+                                                     processor_id,
+                                                     block_region,
+                                                     out_tracker);
         if (status != NV_OK)
             return status;
     }
@@ -714,6 +811,8 @@ NV_STATUS uvm_va_block_unset_read_duplication(uvm_va_block_t *va_block,
     NV_STATUS status = NV_OK;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
 
+    UVM_ASSERT(!uvm_va_block_is_hmm(va_block));
+
     // Restore all SetAccessedBy mappings
     status = UVM_VA_BLOCK_LOCK_RETRY(va_block, &va_block_retry,
                                      va_block_unset_read_duplication_locked(va_block,
@@ -728,7 +827,7 @@ NV_STATUS uvm_va_block_unset_read_duplication(uvm_va_block_t *va_block,
     return status;
 }
 
-static bool read_duplication_is_split_needed(uvm_va_policy_t *policy, void *data)
+static bool read_duplication_is_split_needed(const uvm_va_policy_t *policy, void *data)
 {
     uvm_read_duplication_policy_t new_policy;
 
@@ -740,11 +839,11 @@ static bool read_duplication_is_split_needed(uvm_va_policy_t *policy, void *data
 
 static NV_STATUS read_duplication_set(uvm_va_space_t *va_space, NvU64 base, NvU64 length, bool enable)
 {
-    uvm_va_range_t *va_range, *va_range_last;
     struct mm_struct *mm;
     const NvU64 last_address = base + length - 1;
     NV_STATUS status;
     uvm_read_duplication_policy_t new_policy;
+    uvm_api_range_type_t type;
 
     UVM_ASSERT(va_space);
 
@@ -752,11 +851,13 @@ static NV_STATUS read_duplication_set(uvm_va_space_t *va_space, NvU64 base, NvU6
     mm = uvm_va_space_mm_or_current_retain_lock(va_space);
     uvm_va_space_down_write(va_space);
 
-    status = uvm_api_range_type_check(va_space, mm, base, length);
-    if (status != NV_OK) {
-        if (status == NV_WARN_NOTHING_TO_DO)
-            status = NV_OK;
-
+    type = uvm_api_range_type_check(va_space, mm, base, length);
+    if (type == UVM_API_RANGE_TYPE_INVALID) {
+        status = NV_ERR_INVALID_ADDRESS;
+        goto done;
+    }
+    else if (type == UVM_API_RANGE_TYPE_ATS) {
+        status = NV_OK;
         goto done;
     }
 
@@ -771,44 +872,45 @@ static NV_STATUS read_duplication_set(uvm_va_space_t *va_space, NvU64 base, NvU6
     if (status != NV_OK)
         goto done;
 
-    va_range_last = NULL;
-    uvm_for_each_managed_va_range_in_contig(va_range, va_space, base, last_address) {
-        va_range_last = va_range;
+    if (type == UVM_API_RANGE_TYPE_MANAGED) {
+        uvm_va_range_managed_t *managed_range;
+        uvm_va_range_managed_t *managed_range_last = NULL;
 
-        // If we didn't split the ends, check that they match
-        if (va_range->node.start < base || va_range->node.end > last_address)
-            UVM_ASSERT(uvm_va_range_get_policy(va_range)->read_duplication == new_policy);
+        uvm_for_each_va_range_managed_in_contig(managed_range, va_space, base, last_address) {
+            managed_range_last = managed_range;
 
-        // If the va_space cannot currently read duplicate, only change the user
-        // state. All memory should already have read duplication unset.
-        if (uvm_va_space_can_read_duplicate(va_space, NULL)) {
+            // If we didn't split the ends, check that they match
+            if (managed_range->va_range.node.start < base || managed_range->va_range.node.end > last_address)
+                UVM_ASSERT(managed_range->policy.read_duplication == new_policy);
 
-            // Handle SetAccessedBy mappings
-            if (new_policy == UVM_READ_DUPLICATION_ENABLED) {
-                status = uvm_va_range_set_read_duplication(va_range, mm);
-                if (status != NV_OK)
-                    goto done;
-            }
-            else {
-                // If unsetting read duplication fails, the return status is
-                // not propagated back to the caller
-                (void)uvm_va_range_unset_read_duplication(va_range, mm);
+            // If the va_space cannot currently read duplicate, only change the user
+            // state. All memory should already have read duplication unset.
+            if (uvm_va_space_can_read_duplicate(va_space, NULL)) {
+
+                // Handle SetAccessedBy mappings
+                if (new_policy == UVM_READ_DUPLICATION_ENABLED) {
+                    status = uvm_va_range_set_read_duplication(managed_range, mm);
+                    if (status != NV_OK)
+                        goto done;
+                }
+                else {
+                    // If unsetting read duplication fails, the return status is
+                    // not propagated back to the caller
+                    (void)uvm_va_range_unset_read_duplication(managed_range, mm);
+                }
             }
+
+            managed_range->policy.read_duplication = new_policy;
         }
 
-        uvm_va_range_get_policy(va_range)->read_duplication = new_policy;
+        UVM_ASSERT(managed_range_last);
+        UVM_ASSERT(managed_range_last->va_range.node.end >= last_address);
     }
-
-    if (va_range_last) {
-        UVM_ASSERT(va_range_last->node.end >= last_address);
-        goto done;
+    else {
+        UVM_ASSERT(type == UVM_API_RANGE_TYPE_HMM);
+        status = uvm_hmm_set_read_duplication(va_space, new_policy, base, last_address);
     }
 
-    status = uvm_hmm_set_read_duplication(va_space,
-                                          new_policy,
-                                          base,
-                                          last_address);
-
 done:
     uvm_va_space_up_write(va_space);
     uvm_va_space_mm_or_current_release_unlock(va_space, mm);
@@ -855,20 +957,16 @@ static NV_STATUS system_wide_atomics_set(uvm_va_space_t *va_space, const NvProce
 
     already_enabled = uvm_processor_mask_test(&va_space->system_wide_atomics_enabled_processors, gpu->id);
     if (enable && !already_enabled) {
-        uvm_va_range_t *va_range;
+        uvm_va_range_managed_t *managed_range;
         uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
         uvm_va_block_context_t *va_block_context = uvm_va_space_block_context(va_space, NULL);
         NV_STATUS tracker_status;
 
         // Revoke atomic mappings from the calling GPU
-        uvm_for_each_va_range(va_range, va_space) {
+        uvm_for_each_va_range_managed(managed_range, va_space) {
             uvm_va_block_t *va_block;
 
-            if (va_range->type != UVM_VA_RANGE_TYPE_MANAGED)
-                continue;
-
-            va_block_context->policy = uvm_va_range_get_policy(va_range);
-            for_each_va_block_in_va_range(va_range, va_block) {
+            for_each_va_block_in_va_range(managed_range, va_block) {
                 uvm_page_mask_t *non_resident_pages = &va_block_context->caller_page_mask;
 
                 uvm_mutex_lock(&va_block->lock);
