@@ -260,6 +260,12 @@ namespace DisplayPort
         // Flag to check if the system is UEFI.
         bool        bIsUefiSystem;
 
+        //
+        // Flag to ensure we take into account that
+        // Displayport++ supports HDMI as well.
+        //
+        bool        bHDMIOnDPPlusPlus;
+
         bool        bSkipResetLinkStateDuringPlug;
 
         // Flag to check if LT should be skipped.
