@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2010-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2010-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,12 +34,19 @@
 #include "dp_merger.h"
 #include "dp_list.h"
 #include "dp_tracing.h"
+#include "dp_printf.h"
 
 using namespace DisplayPort;
 namespace DisplayPort
 {
     GenericMessageCompletion::GenericMessageCompletion() :
-        failed(false), completed(false) {}
+        failed(false), completed(false)
+    {
+        // Initialize nakData seperately.
+        nakData.reason      = NakUndefined;
+        nakData.nak_data    = 0;
+        // nakData.guid is initalized in its own constructor.
+    }
     void GenericMessageCompletion::messageFailed(MessageManager::Message * from, NakData * data)
     {
         nakData = *data;
@@ -63,15 +70,23 @@ bool MessageManager::send(MessageManager::Message * message, NakData & nakData)
     DP_USED(sb);
 
     NvU64 startTime, elapsedTime;
-
-    if (bNoReplyTimerForBusyWaiting)
-    {
-        message->bBusyWaiting = true;
-    }
+    message->bBusyWaiting = true;
+    message->setMessagePriority(NV_DP_SBMSG_PRIORITY_LEVEL_1);
     post(message, &completion);
     startTime = timer->getTimeUs();
     do
     {
+        hal->updateDPCDOffline();
+        if (hal->isDpcdOffline())
+        {
+            DP_PRINTF(DP_WARNING, "DP-MM> Device went offline while waiting for reply and so ignoring message %p (ID = %02X, target = %s)",
+                      message, message->requestIdentifier, ((message->state).target).toString(sb));
+            completion.nakData.reason = NakDpcdFail;
+            nakData = completion.nakData;
+            completion.failed = true;
+            break;
+        }
+
         hal->notifyIRQ();
         if (hal->interruptDownReplyReady())
             IRQDownReply();
@@ -81,6 +96,7 @@ bool MessageManager::send(MessageManager::Message * message, NakData & nakData)
             nakData = completion.nakData;
             break;
         }
+
         elapsedTime = timer->getTimeUs() - startTime;
 
         if (elapsedTime > (DPCD_MESSAGE_REPLY_TIMEOUT * 1000))
@@ -183,8 +199,8 @@ void  MessageManager::Message::expired(const void * tag)
     Address::StringBuffer sb;
     DP_USED(sb);
 
-    DP_LOG(("DP-MM> Message transmit time expired on message %p (ID = %02X, target = %s)",
-        (Message*)this, ((Message*)this)->requestIdentifier, (((Message*)this)->state.target).toString(sb)));
+    DP_PRINTF(DP_WARNING, "DP-MM> Message transmit time expired on message %p (ID = %02X, target = %s)",
+          (Message*)this, ((Message*)this)->requestIdentifier, (((Message*)this)->state.target).toString(sb));
 
     Address::NvU32Buffer addrBuffer;
     dpMemZero(addrBuffer, sizeof(addrBuffer));
@@ -430,7 +446,7 @@ void MessageManager::onDownReplyReceived(bool status, EncodedMessage * message)
         }
     }
 
-    DP_LOG(("DPMM> Warning: Unmatched reply message"));
+    DP_PRINTF(DP_WARNING, "DPMM> Warning: Unmatched reply message");
 nextMessage:
     transmitAwaitingUpReplies();
     transmitAwaitingDownRequests();
@@ -461,7 +477,7 @@ MessageManager::~MessageManager()
         for (ListElement * i = notYetSentDownRequest.begin(); i!=notYetSentDownRequest.end(); )
         {
             ListElement * next = i->next;
-            DP_LOG(("Down request message type 0x%x client is not cleaning up.", ((Message *)i)->requestIdentifier));
+            DP_PRINTF(DP_WARNING, "Down request message type 0x%x client is not cleaning up.", ((Message *)i)->requestIdentifier);
             i = next;
         }
     }
@@ -479,7 +495,7 @@ MessageManager::~MessageManager()
         for (ListElement * i = notYetSentUpReply.begin(); i!=notYetSentUpReply.end(); )
         {
             ListElement * next = i->next;
-            DP_LOG(("Up reply message type 0x%x client is not cleaning up.", ((Message *)i)->requestIdentifier));
+            DP_PRINTF(DP_WARNING, "Up reply message type 0x%x client is not cleaning up.", ((Message *)i)->requestIdentifier);
             i = next;
         }
     }
@@ -497,7 +513,7 @@ MessageManager::~MessageManager()
         for (ListElement * i = awaitingReplyDownRequest.begin(); i!=awaitingReplyDownRequest.end(); )
         {
             ListElement * next = i->next;
-            DP_LOG(("Down request message type 0x%x client is not cleaning up.", ((Message *)i)->requestIdentifier));
+            DP_PRINTF(DP_WARNING, "Down request message type 0x%x client is not cleaning up.", ((Message *)i)->requestIdentifier);
             i = next;
         }
     }
@@ -518,7 +534,7 @@ ParseResponseStatus MessageManager::Message::parseResponse(EncodedMessage * mess
     unsigned requestId = reader.readOrDefault(7, 0);
     if (requestId != requestIdentifier)
     {
-        DP_LOG(("DP-MM> Requested = %x Received = %x", requestId, requestIdentifier));
+        DP_PRINTF(DP_NOTICE, "DP-MM> Requested = %x Received = %x", requestId, requestIdentifier);
         DP_ASSERT(0 && "Reply type doesn't match");
         return ParseResponseWrong;
     }
